/* M265 Optical-Computational Processes: Simple Project - whitemist
* Author: Zhenyu Yang
* Date : 2016-4-1
*/

import ddf.minim.spi.*;
import ddf.minim.signals.*;
import ddf.minim.*;
import ddf.minim.analysis.*;
import ddf.minim.ugens.*;
import ddf.minim.effects.*;


Minim minim;
AudioPlayer player;
AudioInput input;


import SimpleOpenNI.*;
SimpleOpenNI  kinect;
PShader blur;

int closestValue;
int closestX;
int closestY;

int canvasWidth  = 640;
int canvasHeight = 480;
int kinectWidth  = 640;
int kinectHeight = 480;

int heartBeatPeriod1 = 80;
int heartBeatPeriod2 = 40;
int heartBeatPeriod3 = 20;
int heartBeatPeriod4 = 15;
int bgmPeriod = 500;
int tempbgmPeriod = 0;

double edgeTransistionRate = 0.15; //transition on the max/min distance edges

PImage maskImage;

// DISTANCE RANGE IN MILLIMETERS (FOR THE FILTER)
int minDistance  = 500;  // 50cm
int maxDistance  = 1500; // 1.5m

int value = 30;
void setup()
{
  
  //Initialization
  minim = new Minim(this);
  size(kinectWidth, kinectHeight, P2D);
  maskImage  = createImage(kinectWidth, kinectHeight, RGB);
  blur = loadShader("blur.glsl");
  kinect = new SimpleOpenNI(this);
  kinect.enableDepth();  
  kinect.enableRGB();
  tempbgmPeriod = bgmPeriod+(int)(Math.random() * 200);  //Initializing temporary backgroundsound period
  
  //Play main background sound in a loop
  AudioPlayer playerloop = minim.loadFile("loop.mp3"); 
  playerloop.play();
  playerloop.loop();
  playerloop.setGain(-4.0); //Adjust volume
}

void draw()
{
  background(255);//Set background as white color.

  //Randomly play temporary background sound effects.
  if(frameCount%tempbgmPeriod==0){
     thread("backgroundSounds");
     tempbgmPeriod = bgmPeriod+(int)(Math.random() * 200);
     println("new tempbgmPeriod = "+tempbgmPeriod);
     }
  closestValue = 8000;
  kinect.update();

  int[] depthValues = kinect.depthMap();

 maskImage.loadPixels();




//Extract a figure of a user from the depthmap. Thresholds are applied.
  // for each row in the depthMap
  for (int y = 0; y < 480; y++) {
    // look at each pixel in the row
    for (int x = 0; x < 640; x++) {
      // pull out the corresponding value from the depth array
      int i = x + y * 640;
      int currentDepthValue = depthValues[i];
      // if that pixel is the closest one we've seen so far
      if (currentDepthValue > 0 && currentDepthValue < closestValue) {
        // save its value
        closestValue = currentDepthValue;
        // and save its position (both X and Y coordinates)
        closestX = x;
        closestY = y;
      }
      if (depthValues[i] > minDistance && depthValues[i] < minDistance*(1+edgeTransistionRate)){
        // IN RANGE: WHITE PIXEL
        maskImage.pixels[i] = color(255-(int)(255*(depthValues[i]-minDistance)/(minDistance*edgeTransistionRate)));
      }
      else if (depthValues[i] < maxDistance*(1-edgeTransistionRate) && depthValues[i] >= minDistance*(1+edgeTransistionRate)){
        maskImage.pixels[i] = color(0);
      }
      else if (depthValues[i] >= maxDistance*(1-edgeTransistionRate) && depthValues[i] < maxDistance){
        maskImage.pixels[i] = color(255-(int)(255*(maxDistance - depthValues[i])/(maxDistance*edgeTransistionRate)));
      }
      else
      maskImage.pixels[i] = color(255);
    }
  }
  maskImage.updatePixels();
  image(maskImage, 0, 0);
  distance2BlurWithFastSpeed(closestValue); //Processing the maskImage
  thread("heartBeat");  //Open a new thread to play heart beat sounds
}

//Image blur processing
void distance2BlurOnGPU(int distance) {
  //println("closestValue = "+closestValue);

  if (distance<=maxDistance&&distance>minDistance)
    multiBlur((distance-minDistance)/20);
  else if (distance<=minDistance)
    multiBlur(0);
  else
    multiBlur((maxDistance-minDistance)/20);
}

void distance2BlurWithFastSpeed(int distance) {
  //println("closestValue = "+closestValue);

  if (distance<=maxDistance&&distance>minDistance)
    superFastBlur((distance-minDistance)/25);
  else if (distance<=minDistance)
    superFastBlur(0);
  else
    superFastBlur((maxDistance-minDistance)/25);
}


void multiBlur(int n) { //a=src, b=dest img
  for (int i = 0; i<n; i++)
    filter(blur);
}



//Sound effects
void heartBeat() {
int distance = closestValue;
  if (minDistance<distance&&distance<=minDistance+(maxDistance-minDistance)/4){
     if(frameCount%heartBeatPeriod4==0){
     player = minim.loadFile("heart3.wav");
     player.play();
     println("heart3.wav");
     }
  }
  else if (minDistance+(maxDistance-minDistance)/4<distance&&distance<=minDistance+((maxDistance-minDistance)*2)/4){
     if(frameCount%heartBeatPeriod3==0){
     player = minim.loadFile("heart3.wav");
     player.play();
     println("heart3.wav");
     }
  }
  else if (minDistance+((maxDistance-minDistance)*2)/4<distance&&distance<=minDistance+((maxDistance-minDistance)*3)/4){
     if(frameCount%heartBeatPeriod2==0){
     player = minim.loadFile("heart2.wav");
     player.play();
          println("heart2.wav");
     }
  }
  else if (minDistance+((maxDistance-minDistance)*3)/4<distance&&distance<=maxDistance){
     if(frameCount%heartBeatPeriod1==0){
     player = minim.loadFile("heart1.wav");
     player.play();
     player.setGain(+6.0); //volume
     println("heart1.wav");
     }
  }
}

void backgroundSounds(){
  player = minim.loadFile("bgm"+(1+(int)(Math.random() * 3))+".mp3");
  
  player.play();
  player.setGain(-10.0); //volume
  println("backgroundSounds");
  
}


//Keyboard interation
void keyPressed() {
  if (key == 's') {
    value ++;
    println("s");
    println("value = "+value);
  } else if (key == 'd') {
    if(value>0)
    value --;
    println("value = "+value);
    println("d");
  }
}


//Mouse interaction
void mousePressed() {
  color c = get(mouseX, mouseY);
  println("r: " + red(c) + " g: " + green(c) + " b: " + blue(c));
}
